import * as React from 'react';
import { TextControl } from '@tesseract-design/web-base';
import plugin from 'tailwindcss/plugin';
import clsx from 'clsx';

/**
 * Derived HTML element of the {@link DropdownSelect} component.
 */
export type DropdownSelectDerivedElement = HTMLSelectElement;

/**
 * Props of the {@link DropdownSelect} component.
 */
export interface DropdownSelectProps extends Omit<React.HTMLProps<DropdownSelectDerivedElement>, 'size' | 'type' | 'label' | 'list' | 'multiple'> {
  /**
   * Short textual description indicating the nature of the component's value.
   */
  label?: React.ReactNode,
  /**
   * Short textual description as guidelines for valid input values.
   */
  hint?: React.ReactNode,
  /**
   * Size of the component.
   */
  size?: TextControl.Size,
  /**
   * Should the component display a border?
   */
  border?: boolean,
  /**
   * Should the component occupy the whole width of its parent?
   */
  block?: boolean,
  /**
   * Style of the component.
   */
  variant?: TextControl.Variant,
  /**
   * Is the label hidden?
   */
  hiddenLabel?: boolean,
}

export const dropdownSelectPlugin = plugin(({ addComponents }) => {
  addComponents({
    '.dropdown-select': {
      '& optgroup': {
        'color': 'rgb(var(--color-positive) / 50%)',
        'text-transform': 'uppercase',
        'font-size': '0.75em',
        'margin-top': '0.5rem',
        'user-select': 'none',
      },
      '& optgroup > option': {
        'color': 'rgb(var(--color-positive))',
        'text-transform': 'none',
        'font-size': '1.333333em',
      },
      '& option': {
        'user-select': 'none',
      },
    },
  });
});

/**
 * Component for selecting a single value from a dropdown.
 */
export const DropdownSelect = React.forwardRef<DropdownSelectDerivedElement, DropdownSelectProps>((
  {
    label,
    hint,
    size = 'medium' as const,
    border = false as const,
    block = false as const,
    variant = 'default' as const,
    hiddenLabel = false as const,
    className,
    children,
    id: idProp,
    style,
    ...etcProps
  }: DropdownSelectProps,
  forwardedRef,
) => {
  const labelId = React.useId();
  const defaultId = React.useId();
  const id = idProp ?? defaultId;

  return (
    <div
      className={clsx(
        'relative rounded ring-secondary/50 min-w-48 group has-[:disabled]:opacity-50',
        'focus-within:ring-4',
        {
          'block': block,
          'inline-block align-middle': !block,
        },
        className,
      )}
      data-testid="base"
      style={style}
    >
      {label && (
        <>
          <label
            htmlFor={id}
            data-testid="label"
            id={labelId}
            className={clsx(
              'absolute z-[1] w-full top-0.5 left-0 pointer-events-none pl-1 text-xxs font-bold group-focus-within:text-secondary text-primary leading-none bg-negative select-none',
              {
                'sr-only': hiddenLabel,
              },
              {
                'pr-10': size === 'small',
                'pr-12': size === 'medium',
                'pr-16': size === 'large',
              },
            )}
          >
            <span className="block w-full whitespace-nowrap h-[1.1em] overflow-hidden text-ellipsis">
              {label}
            </span>
          </label>
          {' '}
        </>
      )}
      <select
        {...etcProps}
        ref={forwardedRef}
        id={id}
        aria-labelledby={labelId}
        data-testid="input"
        role="combobox"
        className={clsx(
          'dropdown-select bg-negative rounded-inherit w-full peer block appearance-none cursor-pointer select-none font-inherit',
          'focus:outline-0',
          'disabled:opacity-50 disabled:cursor-not-allowed',
          {
            'pl-4': variant === 'default',
            'pl-1.5 pt-4': variant === 'alternate',
          },
          {
            'pr-10 h-10 text-xxs': size === 'small',
            'pr-12 h-12 text-xs': size === 'medium',
            'pr-16 h-16': size === 'large',
          },
        )}
      >
        {children}
      </select>
      {hint && (
        <div
          data-testid="hint"
          className={clsx(
            'absolute left-0 px-1 pointer-events-none text-xxs leading-none w-full bg-negative select-none',
            {
              'bottom-0 pl-4 pb-1': variant === 'default',
              'top-0.5': variant === 'alternate',
            },
            {
              'pt-2': variant === 'alternate' && size === 'small',
              'pt-3': variant === 'alternate' && size !== 'small',
            },
            {
              'pr-10': size === 'small',
              'pr-12': size === 'medium',
              'pr-16': size === 'large',
            },
          )}
        >
          <div
            className="opacity-50 whitespace-nowrap w-full h-[1.1em] overflow-hidden text-ellipsis"
          >
            {hint}
          </div>
        </div>
      )}
      <div
        data-testid="indicator"
        className={clsx(
          'text-center flex items-center justify-center aspect-square absolute bottom-0 right-0 pointer-events-none select-none text-primary peer-focus:text-secondary',
          {
            'w-10': size === 'small',
            'w-12': size === 'medium',
            'w-16': size === 'large',
          },
        )}
      >
        <svg
          className="w-6 h-6 fill-none stroke-current stroke-2 linejoin-round linecap-round"
          viewBox="0 0 24 24"
          role="presentation"
        >
          <polyline points="6 9 12 15 18 9" />
        </svg>
      </div>
      {border && (
        <span
          data-testid="border"
          className="absolute z-[1] inset-0 rounded-inherit border-2 border-primary pointer-events-none peer-focus:border-secondary"
        />
      )}
    </div>
  );
});

DropdownSelect.displayName = 'DropdownSelect' as const;

DropdownSelect.defaultProps = {
  label: undefined,
  hint: undefined,
  size: 'medium' as const,
  border: false as const,
  block: false as const,
  variant: 'default' as const,
  hiddenLabel: false as const,
};
