import * as React from 'react';
import { delegateTriggerEvent } from '../event';

type ValueType = string | number | readonly string[];

interface HTMLElementWithValue extends HTMLElement {
  value: ValueType;
}

export interface UseProxyInputOptions<Element extends HTMLElementWithValue, Value = ValueType> {
  forwardedRef: React.Ref<Element>;
  valueSetterFn: (value: Value) => void;
  transformChangeHandlerArgs?: (...e: unknown[]) => Value;
}

export const useProxyInput = <
  T extends HTMLElementWithValue = HTMLElementWithValue,
  V = ValueType,
>(options: UseProxyInputOptions<T, V>) => {
  const {
    forwardedRef,
    valueSetterFn,
    transformChangeHandlerArgs = (e) => e as V,
  } = options;
  const defaultRef = React.useRef<T>(null);

  React.useEffect(() => {
    const { current: currentRaw } = defaultRef;
    const current = currentRaw as unknown as T;
    const forwardedRefProxy = new Proxy(current, {
      get(target, prop, receiver) {
        return Reflect.get(target, prop, receiver) as unknown;
      },
      set(target, prop, newValue, receiver) {
        if (prop === 'value') {
          const thisNewValue = newValue as V;
          const actualNewValue = (
            Array.isArray(thisNewValue) ? thisNewValue.join('') : thisNewValue?.toString() ?? ''
          ) as V;
          valueSetterFn(actualNewValue);
          current.value = actualNewValue as string;
          return true;
        }
        return Reflect.set(target, prop, newValue, receiver);
      },
    }) as unknown as T;

    if (typeof forwardedRef === 'function') {
      forwardedRef(forwardedRefProxy);
      return;
    }

    if (typeof forwardedRef === 'object' && forwardedRef) {
      const mutableForwardedRef = forwardedRef as React.MutableRefObject<T>;
      mutableForwardedRef.current = forwardedRefProxy;
    }
  });

  const handleChange = React.useCallback((e: unknown) => {
    if (!(typeof defaultRef === 'object' && defaultRef)) {
      return;
    }
    const { current: input } = defaultRef;
    if (!input) {
      return;
    }
    valueSetterFn(transformChangeHandlerArgs(e));
    delegateTriggerEvent('change', input, e);
  }, [valueSetterFn, transformChangeHandlerArgs, defaultRef]);

  return React.useMemo(() => ({
    defaultRef,
    handleChange,
  }), [
    defaultRef,
    handleChange,
  ]);
};
